@@ -289,7 +289,7 @@ class GroupedWindowListApplet extends Applet.Applet {
 
         this.actor.set_style_class_name('grouped-window-list-box');
         this.state.set({appletActor: this.actor});
-        this.on_orientation_changed(null);
+        this.on_orientation_changed(orientation);
 
         this.getAutoStartApps();
         this.onSwitchWorkspace = throttle(this.onSwitchWorkspace, 35, false); //Note: causes a 35ms delay in execution
@@ -304,10 +304,6 @@ class GroupedWindowListApplet extends Applet.Applet {
         this.signals.connect(global.display, 'window-demands-attention', (...args) => this.updateAttentionState(...args));
         this.signals.connect(global.display, 'window-created', (...args) => this.onWindowCreated(...args));
         this.signals.connect(global.settings, 'changed::panel-edit-mode', (...args) => this.on_panel_edit_mode_changed(...args));
-        this.signals.connect(Main.overview, 'showing', (...args) => this.onOverviewShow(...args));
-        this.signals.connect(Main.overview, 'hiding', (...args) => this.onOverviewHide(...args));
-        this.signals.connect(Main.expo, 'showing', (...args) => this.onOverviewShow(...args));
-        this.signals.connect(Main.expo, 'hiding', (...args) => this.onOverviewHide(...args));
         this.signals.connect(Main.themeManager, 'theme-set', (...args) => this.refreshCurrentAppList(...args));
     }
 
@@ -407,12 +403,11 @@ class GroupedWindowListApplet extends Applet.Applet {
     }
 
     on_orientation_changed(orientation) {
-        if (orientation) {
-            this.state.set({
-                orientation: orientation,
-                isHorizontal: orientation === St.Side.TOP || orientation === St.Side.BOTTOM
-            });
-        }
+        this.state.set({
+            orientation: orientation,
+            isHorizontal: orientation === St.Side.TOP || orientation === St.Side.BOTTOM
+        });
+
         if (this.state.isHorizontal) {
             this.actor.remove_style_class_name('vertical');
         } else {
@@ -449,8 +444,10 @@ class GroupedWindowListApplet extends Applet.Applet {
     onWindowMonitorChanged(screen, metaWindow, metaWorkspace) {
         if (this.state.monitorWatchList.length !== this.numberOfMonitors) {
             let appList = this.getCurrentAppList();
-            appList.windowRemoved(metaWorkspace, metaWindow);
-            appList.windowAdded(metaWorkspace, metaWindow);
+            if (appList !== null) {
+                appList.windowRemoved(metaWorkspace, metaWindow);
+                appList.windowAdded(metaWorkspace, metaWindow);
+            }
         }
     }
 
@@ -943,7 +940,7 @@ class GroupedWindowListApplet extends Applet.Applet {
 
     _onSwitchWorkspace() {
         if (!this.state) return;
-        this.state.set({currentWs: global.screen.get_active_workspace_index()});
+        this.state.set({currentWs: global.workspace_manager.get_active_workspace_index()});
         let metaWorkspace = global.screen.get_workspace_by_index(this.state.currentWs);
 
         // If the workspace we switched to isn't in our list,
@@ -966,6 +963,7 @@ class GroupedWindowListApplet extends Applet.Applet {
 
         this.actor.remove_all_children();
         this.actor.add_child(this.appLists[refWorkspace].actor);
+        this.actor.queue_relayout();
     }
 
     onWindowSkipTaskbarChanged(screen, metaWindow) {
@@ -983,14 +981,6 @@ class GroupedWindowListApplet extends Applet.Applet {
         this.state.set({thumbnailCloseButtonOffset: global.ui_scale > 1 ? -10 : 0});
         this.refreshAllAppLists();
     }
-
-    onOverviewShow() {
-        this.actor.hide();
-    }
-
-    onOverviewHide() {
-        this.actor.show();
-    }
 }
 
 function main(metadata, orientation, panel_height, instance_id) {
