@@ -1,5 +1,6 @@
 #!/usr/bin/python3
 
+from functools import lru_cache
 import os
 import re
 import html
@@ -7,6 +8,7 @@ import subprocess
 import gettext
 from html.parser import HTMLParser
 import html.entities as entities
+import locale
 
 import gi
 gi.require_version('Gtk', '3.0')
@@ -29,14 +31,22 @@ ROW_SIZE = 32
 
 UNSAFE_ITEMS = ['spawn_sync', 'spawn_command_line_sync', 'GTop', 'get_file_contents_utf8_sync']
 
-curr_ver = subprocess.check_output(['cinnamon', '--version']).decode("utf-8").splitlines()[0].split(' ')[1]
-curr_ver_elements = curr_ver.split(".")
-curr_ver_major = int(curr_ver_elements[0])
-curr_ver_minor = int(curr_ver_elements[1])
+LANGUAGE_CODE = "C"
+try:
+    LANGUAGE_CODE = locale.getlocale()[0].split("_")[0]
+except:
+    pass
+
+
+@lru_cache(maxsize=None)  # fetch only once
+def get_cinnamon_version():
+    version_str = subprocess.check_output(['cinnamon', '--version'], encoding="utf-8").split()[1]
+    return [int(part) for part in version_str.split(".")]
+
 
 def find_extension_subdir(directory):
-    largest = ['0']
-    curr_a = curr_ver.split('.')
+    largest = [0]
+    curr_a = get_cinnamon_version()
 
     for subdir in os.listdir(directory):
         if not os.path.isdir(os.path.join(directory, subdir)):
@@ -45,15 +55,15 @@ def find_extension_subdir(directory):
         if not re.match(r'^[1-9][0-9]*\.[0-9]+(\.[0-9]+)?$', subdir):
             continue
 
-        subdir_a = subdir.split(".")
+        subdir_a = [int(part) for part in subdir.split(".")]
 
-        if subdir_a < curr_a and largest < subdir_a:
+        if subdir_a <= curr_a and largest < subdir_a:
             largest = subdir_a
 
-    if len(largest) == 1:
+    if largest == [0]:
         return directory
     else:
-        return os.path.join(directory, ".".join(largest))
+        return os.path.join(directory, ".".join(map(str, largest)))
 
 translations = {}
 
@@ -84,10 +94,10 @@ def list_header_func(row, before, user_data):
 
 def filter_row(row, entry):
     search_string = entry.get_text().lower()
-    if search_string in row.name.lower() or search_string in row.description.lower() or search_string.lower() in row.uuid.lower():
-        return True
-    else:
-        return False
+    for row_part in [row.name, row.description, row.uuid, row.author]:
+        if search_string.lower() in row_part.lower():
+            return True
+    return False
 
 def show_prompt(msg, window=None):
     dialog = Gtk.MessageDialog(transient_for = window,
@@ -142,7 +152,7 @@ def sanitize_html(string):
 
 
 class ManageSpicesRow(Gtk.ListBoxRow):
-    def __init__(self, extension_type, metadata, size_group):
+    def __init__(self, extension_type, metadata, size_groups):
         super(ManageSpicesRow, self).__init__()
         self.extension_type = extension_type
         self.metadata = metadata
@@ -155,6 +165,13 @@ class ManageSpicesRow(Gtk.ListBoxRow):
         self.name = translate(self.metadata['uuid'], self.metadata['name'])
         self.description = translate(self.metadata['uuid'], self.metadata['description'])
 
+        self.author = ""
+        if 'author' in metadata:
+            if metadata['author'].lower() != "none" and metadata['author'].lower() != "unknown":
+                self.author = metadata['author']
+
+        icon_path = os.path.join(self.metadata['path'], 'icon.png')
+
         try:
             self.max_instances = int(self.metadata['max-instances'])
             if self.max_instances < -1:
@@ -199,15 +216,30 @@ class ManageSpicesRow(Gtk.ListBoxRow):
         grid.set_column_spacing(15)
         widget.pack_start(grid, True, True, 0)
 
+        enabled_box = Gtk.Box()
+        enabled_box.set_spacing(4)
+        size_groups[0].add_widget(enabled_box)
+        self.enabled_image = Gtk.Image.new_from_icon_name('object-select-symbolic', 2)
+        if self.extension_type == "applet":
+            self.enabled_image.set_tooltip_text(_("This applet is currently enabled"))
+        elif self.extension_type == "desklet":
+            self.enabled_image.set_tooltip_text(_("This desklet is currently enabled"))
+        elif self.extension_type == "extension":
+            self.enabled_image.set_tooltip_text(_("This extension is currently enabled"))
+        self.enabled_image.set_no_show_all(True)
+        enabled_box.pack_end(self.enabled_image, False, False, 0)
+        enabled_box.show()
+        grid.attach(enabled_box, 0, 0, 1, 1)
+
         icon = None
         if 'icon' in self.metadata:
             icon_name = self.metadata['icon']
             if Gtk.IconTheme.get_default().has_icon(icon_name):
                 icon = Gtk.Image.new_from_icon_name(icon_name, 3)
 
-        if icon is None and os.path.exists('%s/icon.png' % self.metadata['path']):
+        if os.path.exists(icon_path):
             try:
-                pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale('%s/icon.png' % self.metadata['path'], 24, 24, True)
+                pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale(icon_path, 24, 24, True)
                 icon = Gtk.Image.new_from_pixbuf(pixbuf)
             except:
                 icon = None
@@ -215,20 +247,33 @@ class ManageSpicesRow(Gtk.ListBoxRow):
         if icon is None:
             icon = Gtk.Image.new_from_icon_name('cs-%ss' % (extension_type), 3)
 
-        grid.attach(icon, 0, 0, 1, 1)
+        grid.attach_next_to(icon, enabled_box, Gtk.PositionType.RIGHT, 1, 1)
 
         desc_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         desc_box.props.hexpand = True
         desc_box.props.halign = Gtk.Align.START
+        desc_box.set_spacing(1)
+
         name_label = Gtk.Label()
         name_markup = GLib.markup_escape_text(self.name)
-        name_label.set_markup('<b>{}</b>'.format(name_markup))
+        if self.author == "":
+            name_label.set_markup('<b>{}</b>'.format(name_markup))
+        else:
+            by_author = _("by %s") % self.author
+            name_label.set_markup('<b>{}</b><small> {}</small>'.format(name_markup, by_author))
         name_label.props.xalign = 0.0
         desc_box.add(name_label)
 
+        uuid_label = Gtk.Label()
+        uuid_markup = GLib.markup_escape_text(self.uuid)
+        uuid_label.set_markup('<small><i>{}</i></small>'.format(uuid_markup))
+        uuid_label.props.xalign = 0.0
+        desc_box.add(uuid_label)
+
         description_label = SettingsLabel()
         description_markup = GLib.markup_escape_text(sanitize_html(self.description))
         description_label.set_markup('<small>{}</small>'.format(description_markup))
+        description_label.set_margin_top(2)
         desc_box.add(description_label)
 
         grid.attach_next_to(desc_box, icon, Gtk.PositionType.RIGHT, 1, 1)
@@ -236,11 +281,12 @@ class ManageSpicesRow(Gtk.ListBoxRow):
         self.status_box = Gtk.Box()
         self.status_box.set_spacing(4)
         grid.attach_next_to(self.status_box, desc_box, Gtk.PositionType.RIGHT, 1, 1)
+        size_groups[1].add_widget(self.status_box)
 
         self.button_box = Gtk.Box()
         self.button_box.set_valign(Gtk.Align.CENTER)
         grid.attach_next_to(self.button_box, self.status_box, Gtk.PositionType.RIGHT, 1, 1)
-        size_group.add_widget(self.button_box)
+        size_groups[2].add_widget(self.button_box)
 
         if self.has_config:
             config_icon = Gtk.Image.new_from_icon_name('system-run-symbolic', 2)
@@ -272,11 +318,11 @@ class ManageSpicesRow(Gtk.ListBoxRow):
         try:
             # Treat "cinnamon-version" as a list of minimum required versions
             # if any version in there is lower than our Cinnamon version, then the spice is compatible.
+            curr_ver = get_cinnamon_version()
+
             for version in self.metadata['cinnamon-version']:
-                elements = version.split(".")
-                major = int(elements[0])
-                minor = int(elements[1])
-                if curr_ver_major > major or (curr_ver_major == major and curr_ver_minor >= minor):
+                spice_ver = [int(part) for part in version.split(".")]
+                if spice_ver[:2] <= curr_ver:
                     # The version is OK, check that we can find the right .js file in the appropriate subdir
                     path = os.path.join(self.metadata['path'], self.extension_type + ".js")
                     if os.path.exists(path):
@@ -284,7 +330,6 @@ class ManageSpicesRow(Gtk.ListBoxRow):
                     else:
                         print ("The %s %s is not properly structured. Path not found: '%s'" % (self.uuid, self.extension_type, path))
                         return False
-                    return True
             print ("The %s %s is not compatible with this version of Cinnamon." % (self.uuid, self.extension_type))
             return False
         except:
@@ -325,14 +370,9 @@ class ManageSpicesRow(Gtk.ListBoxRow):
         self.enabled = enabled
 
         if self.enabled:
-            if self.extension_type == "applet":
-                self.add_status('enabled', 'object-select-symbolic', _("This applet is currently enabled"))
-            elif self.extension_type == "desklet":
-                self.add_status('enabled', 'object-select-symbolic', _("This desklet is currently enabled"))
-            elif self.extension_type == "extension":
-                self.add_status('enabled', 'object-select-symbolic', _("This extension is currently enabled"))
+            self.enabled_image.show()
         else:
-            self.remove_status('enabled')
+            self.enabled_image.hide()
         if self.has_config:
             self.config_button.set_sensitive(enabled)
 
@@ -587,10 +627,11 @@ class ManageSpicesPage(SettingsPage):
 
         self.extension_rows = []
 
-        size_group = Gtk.SizeGroup.new(Gtk.SizeGroupMode.HORIZONTAL)
+        size_groups = [Gtk.SizeGroup.new(Gtk.SizeGroupMode.HORIZONTAL) for i in range(3)]
+
         for uuid, metadata in self.spices.get_installed().items():
             try:
-                extension_row = ManageSpicesRow(self.collection_type, metadata, size_group)
+                extension_row = ManageSpicesRow(self.collection_type, metadata, size_groups)
                 self.list_box.add(extension_row)
                 self.extension_rows.append(extension_row)
                 extension_row.set_enabled(self.spices.get_enabled(uuid))
@@ -626,6 +667,19 @@ class DownloadSpicesRow(Gtk.ListBoxRow):
         self.score = data['score']
         self.timestamp = data['last_edited']
 
+        self.author = ""
+        if 'author_user' in data:
+            if data['author_user'].lower() != "none" and data['author_user'].lower() != "unknown":
+                self.author = data['author_user']
+
+        if 'translations' in data.keys():
+            key = 'name_%s' % LANGUAGE_CODE
+            if key in data['translations'].keys():
+                self.name = data['translations'][key]
+            key = 'description_%s' % LANGUAGE_CODE
+            if key in data['translations'].keys():
+                self.description = data['translations'][key]
+
         self.has_update = False
 
         self.status_ids = {}
@@ -636,22 +690,48 @@ class DownloadSpicesRow(Gtk.ListBoxRow):
         widget.set_spacing(15)
         self.add(widget)
 
+        installed_box = Gtk.Box()
+        installed_box.set_spacing(4)
+        widget.pack_start(installed_box, False, False, 0)
+        size_groups[0].add_widget(installed_box)
+        installed_image = Gtk.Image.new_from_icon_name('object-select-symbolic', 2)
+        installed_box.pack_end(installed_image, False, False, 0)
+        installed_image.set_tooltip_text(_("Installed"))
+        installed_image.set_no_show_all(True)
+        if self.installed:
+            installed_image.show()
+        else:
+            installed_image.hide()
+
         icon = spices.get_icon(uuid)
         widget.pack_start(icon, False, False, 0)
 
         desc_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         desc_box.set_hexpand(True)
         desc_box.set_halign(Gtk.Align.FILL)
+        desc_box.set_spacing(1)
+
         name_label = Gtk.Label()
         name_markup = GLib.markup_escape_text(self.name)
-        name_label.set_markup('<b>{}</b>'.format(name_markup))
+        if self.author == "":
+            name_label.set_markup('<b>{}</b>'.format(name_markup))
+        else:
+            by_author = _("by %s") % self.author
+            name_label.set_markup('<b>{}</b><small> {}</small>'.format(name_markup, by_author))
         name_label.set_hexpand(True)
         name_label.set_halign(Gtk.Align.START)
         desc_box.pack_start(name_label, False, False, 0)
 
+        uuid_label = Gtk.Label()
+        uuid_markup = GLib.markup_escape_text(self.uuid)
+        uuid_label.set_markup('<small><i>{}</i></small>'.format(uuid_markup))
+        uuid_label.props.xalign = 0.0
+        desc_box.add(uuid_label)
+
         description_label = SettingsLabel()
         description_markup = GLib.markup_escape_text(sanitize_html(self.description))
         description_label.set_markup('<small>{}</small>'.format(description_markup))
+        description_label.set_margin_top(2)
         desc_box.pack_start(description_label, False, False, 0)
 
         widget.pack_start(desc_box, True, True, 0)
@@ -662,18 +742,18 @@ class DownloadSpicesRow(Gtk.ListBoxRow):
         score_label = Gtk.Label(self.score)
         score_box.pack_start(score_label, False, False, 5)
         widget.pack_start(score_box, False, False, 0)
-        size_groups[0].add_widget(score_box)
+        size_groups[1].add_widget(score_box)
 
         self.status_box = Gtk.Box()
         self.status_box.set_spacing(4)
         widget.pack_start(self.status_box, False, False, 0)
-        size_groups[1].add_widget(self.status_box)
+        size_groups[2].add_widget(self.status_box)
 
         self.button_box = Gtk.Box()
         self.button_box.set_valign(Gtk.Align.CENTER)
         self.button_box.set_baseline_position(Gtk.BaselinePosition.CENTER)
         widget.pack_start(self.button_box, False, False, 0)
-        size_groups[2].add_widget(self.button_box)
+        size_groups[3].add_widget(self.button_box)
 
         if not self.installed:
             download_button = Gtk.Button.new_from_icon_name('go-down-symbolic', 2)
@@ -687,9 +767,6 @@ class DownloadSpicesRow(Gtk.ListBoxRow):
             download_button.connect('clicked', self.download)
             download_button.set_tooltip_text(_("Update"))
 
-        if self.installed:
-            self.add_status('installed', 'object-select-symbolic', _("Installed"))
-
     def download(self, *args):
         self.spices.install(self.uuid)
 
@@ -725,6 +802,8 @@ class DownloadSpicesPage(SettingsPage):
         self.extension_rows = []
         self._signals = []
 
+        self.initial_refresh_done = False
+
         self.top_box = Gtk.Box()
         self.pack_start(self.top_box, False, False, 10)
 
@@ -924,7 +1003,7 @@ class DownloadSpicesPage(SettingsPage):
                 row.destroy()
             self.extension_rows = []
 
-        size_groups = [Gtk.SizeGroup.new(Gtk.SizeGroupMode.HORIZONTAL) for i in range(3)]
+        size_groups = [Gtk.SizeGroup.new(Gtk.SizeGroupMode.HORIZONTAL) for i in range(4)]
 
         for uuid, data in spices_data.items():
             row = DownloadSpicesRow(uuid, data, self.spices, size_groups)
@@ -955,36 +1034,8 @@ class DownloadSpicesPage(SettingsPage):
         if not self.extension_rows:
             self.build_list()
 
-        if not self.spices.processing_jobs:
-            if (not self.spices.has_cache) or self.spices.get_cache_age() > 7:
-                self.on_cache_outdated()
-
-        self.search_entry.grab_focus()
-
-    def on_cache_outdated(self, *args):
-        infobar = self.infobar_holder.get_child()
-        if not infobar:
-            infobar = Gtk.InfoBar()
-            icon = Gtk.Image.new_from_icon_name("dialog-question-symbolic", Gtk.IconSize.LARGE_TOOLBAR)
-            label = Gtk.Label(_("Your cache is out of date. Would you like to update it now?"))
-            label.set_line_wrap(True)
-
-            infobar.set_message_type(Gtk.MessageType.QUESTION)
-            infobar.get_content_area().pack_start(icon, False, False, 12)
-            infobar.get_content_area().pack_start(label, False, False, 0)
-            infobar.add_button(_("Yes"), Gtk.ResponseType.YES)
-            infobar.add_button(_("No"), Gtk.ResponseType.NO)
-
-            infobar.connect('response', self._on_infobar_response)
-            self.infobar_holder.add(infobar)
-
-        self.infobar_holder.show_all()
-        infobar.set_revealed(True)
-
-    def _on_infobar_response(self, infobar: Gtk.InfoBar, response):
-        if response == Gtk.ResponseType.YES and not self.spices.processing_jobs:
+        if (not self.initial_refresh_done) and (not self.spices.processing_jobs):
+            self.initial_refresh_done = True
             self.spices.refresh_cache()
-            pass
 
-        infobar.set_revealed(False)
         self.search_entry.grab_focus()
